@@ -57,7 +57,6 @@ Street, Fifth Floor, Boston, MA 02110-1335 USA
 #include "backup_copy.h"
 #include "backup_mysql.h"
 #include <btr0btr.h>
-#include "xb0xb.h"
 
 #define ROCKSDB_BACKUP_DIR "#rocksdb"
 
@@ -642,12 +641,10 @@ int
 mkdirp(const char *pathname, int Flags, myf MyFlags)
 {
 	char *parent, *p;
-	int len = strlen(pathname) + 1;
 
 	/* make a parent directory path */
-	if (!(parent= (char *)malloc(len)))
+	if (!(parent= strdup(pathname)))
           return(-1);
-	memcpy(parent, pathname, len);
 
 	for (p = parent + strlen(parent);
 	    !is_path_separator(*p) && p != parent; p--) ;
@@ -867,14 +864,21 @@ datafile_rsync_backup(const char *filepath, bool save_to_list, FILE *f)
 	return(true);
 }
 
-bool backup_file_print_buf(const char *filename, const char *buf, int buf_len)
+
+static
+bool
+backup_file_vprintf(const char *filename, const char *fmt, va_list ap)
 {
 	ds_file_t	*dstfile	= NULL;
 	MY_STAT		 stat;			/* unused for now */
+	char		*buf		= 0;
+	int		 buf_len;
 	const char	*action;
 
 	memset(&stat, 0, sizeof(stat));
 
+	buf_len = vasprintf(&buf, fmt, ap);
+
 	stat.st_size = buf_len;
 	stat.st_mtime = my_time(0);
 
@@ -898,6 +902,7 @@ bool backup_file_print_buf(const char *filename, const char *buf, int buf_len)
 
 	/* close */
 	msg("        ...done");
+	free(buf);
 
 	if (ds_close(dstfile)) {
 		goto error_close;
@@ -906,6 +911,7 @@ bool backup_file_print_buf(const char *filename, const char *buf, int buf_len)
 	return(true);
 
 error:
+	free(buf);
 	if (dstfile != NULL) {
 		ds_close(dstfile);
 	}
@@ -913,22 +919,9 @@ error:
 error_close:
 	msg("Error: backup file failed.");
 	return(false); /*ERROR*/
-
-	return true;
-};
-
-static
-bool
-backup_file_vprintf(const char *filename, const char *fmt, va_list ap)
-{
-	char		*buf		= 0;
-	int		 buf_len;
-	buf_len = vasprintf(&buf, fmt, ap);
-	bool result = backup_file_print_buf(filename, buf, buf_len);
-	free(buf);
-	return result;
 }
 
+
 bool
 backup_file_printf(const char *filename, const char *fmt, ...)
 {
@@ -1066,7 +1059,6 @@ copy_file(ds_ctxt_t *datasink,
 	ds_file_t		*dstfile = NULL;
 	datafile_cur_t		 cursor;
 	xb_fil_cur_result_t	 res;
-	DBUG_ASSERT(datasink->datasink->remove);
 	const char	*dst_path =
 		(xtrabackup_copy_back || xtrabackup_move_back)?
 		dst_file_path : trim_dotslash(dst_file_path);
@@ -1092,7 +1084,6 @@ copy_file(ds_ctxt_t *datasink,
 		if (ds_write(dstfile, cursor.buf, cursor.buf_read)) {
 			goto error;
 		}
-		DBUG_EXECUTE_IF("copy_file_error", errno=ENOSPC;goto error;);
 	}
 
 	if (res == XB_FIL_CUR_ERROR) {
@@ -1114,7 +1105,6 @@ copy_file(ds_ctxt_t *datasink,
 error:
 	datafile_close(&cursor);
 	if (dstfile != NULL) {
-		datasink->datasink->remove(dstfile->path);
 		ds_close(dstfile);
 	}
 
@@ -1159,18 +1149,17 @@ move_file(ds_ctxt_t *datasink,
 
 	if (my_rename(src_file_path, dst_file_path_abs, MYF(0)) != 0) {
 		if (my_errno == EXDEV) {
-			/* Fallback to copy/unlink */
-			if(!copy_file(datasink, src_file_path,
-					dst_file_path, thread_n))
-					return false;
+			bool ret;
+			ret = copy_file(datasink, src_file_path,
+					dst_file_path, thread_n);
 			msg(thread_n,"Removing %s", src_file_path);
 			if (unlink(src_file_path) != 0) {
 				my_strerror(errbuf, sizeof(errbuf), errno);
-				msg("Warning: unlink %s failed: %s",
+				msg("Error: unlink %s failed: %s",
 					src_file_path,
 					errbuf);
 			}
-			return true;
+			return(ret);
 		}
 		my_strerror(errbuf, sizeof(errbuf), my_errno);
 		msg("Can not move file %s to %s: %s",
@@ -1454,7 +1443,7 @@ out:
 	return(ret);
 }
 
-void backup_fix_ddl(CorruptedPages &);
+void backup_fix_ddl(void);
 
 lsn_t get_current_lsn(MYSQL *connection)
 {
@@ -1479,7 +1468,7 @@ lsn_t get_current_lsn(MYSQL *connection)
 lsn_t server_lsn_after_lock;
 extern void backup_wait_for_lsn(lsn_t lsn);
 /** Start --backup */
-bool backup_start(CorruptedPages &corrupted_pages)
+bool backup_start()
 {
 	if (!opt_no_lock) {
 		if (opt_safe_slave_backup) {
@@ -1514,7 +1503,7 @@ bool backup_start(CorruptedPages &corrupted_pages)
 
 	msg("Waiting for log copy thread to read lsn %llu", (ulonglong)server_lsn_after_lock);
 	backup_wait_for_lsn(server_lsn_after_lock);
-	backup_fix_ddl(corrupted_pages);
+	backup_fix_ddl();
 
 	// There is no need to stop slave thread before coping non-Innodb data when
 	// --no-lock option is used because --no-lock option requires that no DDL or
