@@ -73,14 +73,12 @@ usage () {
         cat <<EOF
 Usage: $0 [OPTIONS]
   --no-defaults              Don't read the system defaults file
+  --core-file-size=LIMIT     Limit core files to the specified size
   --defaults-file=FILE       Use the specified defaults file
   --defaults-extra-file=FILE Also use defaults from the specified file
-  --defaults-group-suffix=X  Additionally read default groups with X appended
-                             as a suffix
   --ledir=DIRECTORY          Look for mysqld in the specified directory
   --open-files-limit=LIMIT   Limit the number of open files
   --crash-script=FILE        Script to call when mysqld crashes
-  --core-file-size=LIMIT     Limit core files to the specified size
   --timezone=TZ              Set the system timezone
   --malloc-lib=LIB           Preload shared library LIB if available
   --mysqld=FILE              Use the specified file as mysqld
@@ -106,35 +104,6 @@ EOF
         exit 1
 }
 
-my_which ()
-{
-  save_ifs="${IFS-UNSET}"
-  IFS=:
-  ret=0
-  for file
-  do
-    for dir in $PATH
-    do
-      if [ -f "$dir/$file" ]
-      then
-        echo "$dir/$file"
-        continue 2
-      fi
-    done
-
-	ret=1  #signal an error
-	break
-  done
-
-  if [ "$save_ifs" = UNSET ]
-  then
-    unset IFS
-  else
-    IFS="$save_ifs"
-  fi
-  return $ret  # Success
-}
-
 find_in_bin() {
   if test -x "$MY_BASEDIR_VERSION/bin/$1"
   then
@@ -157,7 +126,7 @@ log_generic () {
   case $logging in
     init) ;;  # Just echo the message, don't save it anywhere
     file)
-      if [ "$helper_exist" -eq "0" ]; then
+      if [ -n "$helper" ]; then
         echo "$msg" | "$helper" "$user" log "$err_log"
       fi
     ;;
@@ -181,7 +150,7 @@ eval_log_error () {
   local cmd="$1"
   case $logging in
     file)
-     if [ "$helper_exist" -eq "0" ]; then
+     if [ -n "$helper" ]; then
         cmd="$cmd 2>&1 | "`shell_quote_string "$helper"`" $user log "`shell_quote_string "$err_log"`
      fi
      ;;
@@ -223,7 +192,8 @@ wsrep_pick_url() {
 
   log_error "WSREP: 'wsrep_urls' is DEPRECATED! Use wsrep_cluster_address to specify multiple addresses instead."
 
-  if ! which nc >/dev/null; then
+  if ! command -v nc >/dev/null
+  then
     log_error "ERROR: nc tool not found in PATH! Make sure you have it installed."
     return 1
   fi
@@ -553,9 +523,10 @@ fi
 
 helper=`find_in_bin mysqld_safe_helper`
 print_defaults=`find_in_bin my_print_defaults`
+
 # Check if helper exists
-command -v $helper --help >/dev/null 2>&1
-helper_exist=$?
+$helper --help >/dev/null 2>&1 || helper=""
+
 #
 # Second, try to find the data directory
 #
@@ -641,8 +612,7 @@ plugin_dir="${plugin_dir}${PLUGIN_VARIANT}"
 # Ensure that 'logger' exists, if it's requested
 if [ $want_syslog -eq 1 ]
 then
-  my_which logger > /dev/null 2>&1
-  if [ $? -ne 0 ]
+  if ! command -v logger > /dev/null
   then
     log_error "--syslog requested, but no 'logger' program found.  Please ensure that 'logger' is in your PATH, or do not specify the --syslog option to mysqld_safe."
     exit 1
@@ -873,7 +843,7 @@ fi
 if @TARGET_LINUX@ && test $flush_caches -eq 1
 then
   # Locate sync, ensure it exists.
-  if ! my_which sync > /dev/null 2>&1
+  if ! command -v sync > /dev/null
   then
     log_error "sync command not found, required for --flush-caches"
     exit 1
@@ -885,7 +855,7 @@ then
   fi
 
   # Locate sysctl, ensure it exists.
-  if ! my_which sysctl > /dev/null 2>&1
+  if ! command -v sysctl > /dev/null
   then
     log_error "sysctl command not found, required for --flush-caches"
     exit 1
@@ -929,7 +899,7 @@ cmd="`mysqld_ld_preload_text`$NOHUP_NICENESS"
 if @TARGET_LINUX@ && test $numa_interleave -eq 1
 then
   # Locate numactl, ensure it exists.
-  if ! my_which numactl > /dev/null 2>&1
+  if ! command -v numactl > /dev/null
   then
     log_error "numactl command not found, required for --numa-interleave"
     exit 1
@@ -963,6 +933,7 @@ fi
 
 # Avoid 'nohup: ignoring input' warning
 test -n "$NOHUP_NICENESS" && cmd="$cmd < /dev/null"
+
 log_notice "Starting $MYSQLD daemon with databases from $DATADIR"
 
 # variable to track the current number of "fast" (a.k.a. subsecond) restarts
