@@ -25,7 +25,7 @@
 #pragma implementation				// gcc: Class implementation
 #endif
 
-#include <my_global.h>
+#include "mariadb.h"
 #include "sql_priv.h"
 #include "protocol.h"
 #include "sql_class.h"                          // THD
@@ -87,7 +87,7 @@ bool Protocol_binary::net_store_data_cs(const uchar *from, size_t length,
 {
   uint dummy_errors;
   /* Calculate maxumum possible result length */
-  uint conv_length= to_cs->mbmaxlen * length / from_cs->mbminlen;
+  size_t conv_length= to_cs->mbmaxlen * length / from_cs->mbminlen;
 
   if (conv_length > 250)
   {
@@ -107,8 +107,8 @@ bool Protocol_binary::net_store_data_cs(const uchar *from, size_t length,
             net_store_data((const uchar*) convert->ptr(), convert->length()));
   }
 
-  ulong packet_length= packet->length();
-  ulong new_length= packet_length + conv_length + 1;
+  size_t packet_length= packet->length();
+  size_t new_length= packet_length + conv_length + 1;
 
   if (new_length > packet->alloced_length() && packet->realloc(new_length))
     return 1;
@@ -218,6 +218,8 @@ net_send_ok(THD *thd,
   NET *net= &thd->net;
   StringBuffer<MYSQL_ERRMSG_SIZE + 10> store;
 
+  bool state_changed= false;
+
   bool error= FALSE;
   DBUG_ENTER("net_send_ok");
 
@@ -244,11 +246,6 @@ net_send_ok(THD *thd,
   /* last insert id */
   store.q_net_store_length(id);
 
-  /* if client has not session tracking capability, don't send state change flag*/
-  if (!(thd->client_capabilities & CLIENT_SESSION_TRACK)) {
-    server_status &= ~SERVER_SESSION_STATE_CHANGED;
-  }
-
   if (thd->client_capabilities & CLIENT_PROTOCOL_41)
   {
     DBUG_PRINT("info",
@@ -269,25 +266,31 @@ net_send_ok(THD *thd,
   }
   thd->get_stmt_da()->set_overwrite_status(true);
 
-  if ((server_status & SERVER_SESSION_STATE_CHANGED) || (message && message[0]))
+  state_changed=
+    (thd->client_capabilities & CLIENT_SESSION_TRACK) &&
+    (server_status & SERVER_SESSION_STATE_CHANGED);
+
+  if (state_changed || (message && message[0]))
   {
     DBUG_ASSERT(safe_strlen(message) <= MYSQL_ERRMSG_SIZE);
     store.q_net_store_data((uchar*) safe_str(message), safe_strlen(message));
   }
 
-  if (unlikely(server_status & SERVER_SESSION_STATE_CHANGED))
+  if (unlikely(state_changed))
   {
     store.set_charset(thd->variables.collation_database);
+
     thd->session_tracker.store(thd, &store);
-    thd->server_status&= ~SERVER_SESSION_STATE_CHANGED;
   }
 
   DBUG_ASSERT(store.length() <= MAX_PACKET_LENGTH);
 
   error= my_net_write(net, (const unsigned char*)store.ptr(), store.length());
-  if (!error && (!skip_flush || is_eof))
+  if (likely(!error) && (!skip_flush || is_eof))
     error= net_flush(net);
 
+  thd->server_status&= ~SERVER_SESSION_STATE_CHANGED;
+
   thd->get_stmt_da()->set_overwrite_status(false);
   DBUG_PRINT("info", ("OK sent, so no more error sending allowed"));
 
@@ -347,7 +350,7 @@ net_send_eof(THD *thd, uint server_status, uint statement_warn_count)
   {
     thd->get_stmt_da()->set_overwrite_status(true);
     error= write_eof_packet(thd, net, server_status, statement_warn_count);
-    if (!error)
+    if (likely(!error))
       error= net_flush(net);
     thd->get_stmt_da()->set_overwrite_status(false);
     DBUG_PRINT("info", ("EOF sent, so no more error sending allowed"));
@@ -391,7 +394,7 @@ static bool write_eof_packet(THD *thd, NET *net,
       because if 'is_fatal_error' is set the server is not going to execute
       other queries (see the if test in dispatch_command / COM_QUERY)
     */
-    if (thd->is_fatal_error)
+    if (unlikely(thd->is_fatal_error))
       server_status&= ~SERVER_MORE_RESULTS_EXISTS;
     int2store(buff + 3, server_status);
     error= my_net_write(net, buff, 5);
@@ -467,12 +470,8 @@ bool net_send_error_packet(THD *thd, uint sql_errno, const char *err,
     coming from server to have seq_no > 0, due to missing awareness
     of "out-of-band" operations. Make these clients happy.
   */
-  if (!net->pkt_nr &&
-      (sql_errno == ER_CONNECTION_KILLED || sql_errno == ER_SERVER_SHUTDOWN ||
-       sql_errno == ER_QUERY_INTERRUPTED))
-  {
-    net->pkt_nr= 1;
-  }
+  if (!net->pkt_nr)
+   net->pkt_nr= 1;
 
   ret= net_write_command(net,(uchar) 255, (uchar*) "", 0, (uchar*) buff,
                          length);
@@ -493,8 +492,9 @@ bool net_send_error_packet(THD *thd, uint sql_errno, const char *err,
   - ulonglong for bigger numbers.
 */
 
-static uchar *net_store_length_fast(uchar *packet, uint length)
+static uchar *net_store_length_fast(uchar *packet, size_t length)
 {
+  DBUG_ASSERT(length < UINT_MAX16);
   if (length < 251)
   {
     *packet=(uchar) length;
@@ -602,7 +602,7 @@ void Protocol::end_statement()
                    thd->get_stmt_da()->skip_flush());
     break;
   }
-  if (!error)
+  if (likely(!error))
     thd->get_stmt_da()->set_is_sent(true);
   DBUG_VOID_RETURN;
 }
@@ -674,7 +674,7 @@ void net_send_progress_packet(THD *thd)
 {
   uchar buff[200], *pos;
   const char *proc_info= thd->proc_info ? thd->proc_info : "";
-  uint length= strlen(proc_info);
+  size_t length= strlen(proc_info);
   ulonglong progress;
   DBUG_ENTER("net_send_progress_packet");
 
@@ -805,8 +805,7 @@ bool Protocol::send_result_set_metadata(List<Item> *list, uint flags)
 {
   List_iterator_fast<Item> it(*list);
   Item *item;
-  uchar buff[MAX_FIELD_WIDTH];
-  String tmp((char*) buff,sizeof(buff),&my_charset_bin);
+  ValueBuffer<MAX_FIELD_WIDTH> tmp;
   Protocol_text prot(thd);
   String *local_packet= prot.storage_packet();
   CHARSET_INFO *thd_charset= thd->variables.character_set_results;
@@ -814,7 +813,9 @@ bool Protocol::send_result_set_metadata(List<Item> *list, uint flags)
 
   if (flags & SEND_NUM_ROWS)
   {				// Packet with number of elements
+    uchar buff[MAX_INT_WIDTH];
     uchar *pos= net_store_length(buff, list->elements);
+    DBUG_ASSERT(pos <= buff + sizeof(buff));
     if (my_net_write(&thd->net, buff, (size_t) (pos-buff)))
       DBUG_RETURN(1);
   }
@@ -833,7 +834,7 @@ bool Protocol::send_result_set_metadata(List<Item> *list, uint flags)
     char *pos;
     CHARSET_INFO *cs= system_charset_info;
     Send_field field;
-    item->make_field(thd, &field);
+    item->make_send_field(thd, &field);
 
     /* limit number of decimals for float and double */
     if (field.type == MYSQL_TYPE_FLOAT || field.type == MYSQL_TYPE_DOUBLE)
@@ -854,9 +855,9 @@ bool Protocol::send_result_set_metadata(List<Item> *list, uint flags)
 		     cs, thd_charset) ||
 	  prot.store(field.org_table_name, (uint) strlen(field.org_table_name),
 		     cs, thd_charset) ||
-	  prot.store(field.col_name, (uint) strlen(field.col_name),
+	  prot.store(field.col_name.str, (uint) field.col_name.length,
 		     cs, thd_charset) ||
-	  prot.store(field.org_col_name, (uint) strlen(field.org_col_name),
+	  prot.store(field.org_col_name.str, (uint) field.org_col_name.length,
 		     cs, thd_charset) ||
 	  local_packet->realloc(local_packet->length()+12))
 	goto err;
@@ -912,7 +913,7 @@ bool Protocol::send_result_set_metadata(List<Item> *list, uint flags)
     {
       if (prot.store(field.table_name, (uint) strlen(field.table_name),
 		     cs, thd_charset) ||
-	  prot.store(field.col_name, (uint) strlen(field.col_name),
+	  prot.store(field.col_name.str, (uint) field.col_name.length,
 		     cs, thd_charset) ||
 	  local_packet->realloc(local_packet->length()+10))
 	goto err;
@@ -982,29 +983,28 @@ bool Protocol::write()
 
 bool Protocol::send_result_set_row(List<Item> *row_items)
 {
-  char buffer[MAX_FIELD_WIDTH];
-  String str_buffer(buffer, sizeof (buffer), &my_charset_bin);
   List_iterator_fast<Item> it(*row_items);
 
   DBUG_ENTER("Protocol::send_result_set_row");
 
   for (Item *item= it++; item; item= it++)
   {
-    if (item->send(this, &str_buffer))
+    /*
+      ValueBuffer::m_string can be altered during Item::send().
+      It's important to declare value_buffer inside the loop,
+      to have ValueBuffer::m_string point to ValueBuffer::buffer
+      on every iteration.
+    */
+    ValueBuffer<MAX_FIELD_WIDTH> value_buffer;
+    if (item->send(this, &value_buffer))
     {
       // If we're out of memory, reclaim some, to help us recover.
       this->free();
       DBUG_RETURN(TRUE);
     }
     /* Item::send() may generate an error. If so, abort the loop. */
-    if (thd->is_error())
+    if (unlikely(thd->is_error()))
       DBUG_RETURN(TRUE);
-
-    /*
-      Reset str_buffer to its original state, as it may have been altered in
-      Item::send().
-    */
-    str_buffer.set(buffer, sizeof(buffer), &my_charset_bin);
   }
 
   DBUG_RETURN(FALSE);
@@ -1029,7 +1029,7 @@ bool Protocol::store(const char *from, CHARSET_INFO *cs)
 {
   if (!from)
     return store_null();
-  uint length= strlen(from);
+  size_t length= strlen(from);
   return store(from, length, cs);
 }
 
@@ -1130,7 +1130,7 @@ bool Protocol_text::store(const char *from, size_t length,
 #ifndef DBUG_OFF
   DBUG_PRINT("info", ("Protocol_text::store field %u (%u): %.*s", field_pos,
                       field_count, (int) length, (length == 0 ? "" : from)));
-  DBUG_ASSERT(field_pos < field_count);
+  DBUG_ASSERT(field_types == 0 || field_pos < field_count);
   DBUG_ASSERT(field_types == 0 ||
 	      field_types[field_pos] == MYSQL_TYPE_DECIMAL ||
               field_types[field_pos] == MYSQL_TYPE_BIT ||
@@ -1222,7 +1222,7 @@ bool Protocol_text::store(float from, uint32 decimals, String *buffer)
 	      field_types[field_pos] == MYSQL_TYPE_FLOAT);
   field_pos++;
 #endif
-  buffer->set_real((double) from, decimals, thd->charset());
+  Float(from).to_string(buffer, decimals);
   return net_store_data((uchar*) buffer->ptr(), buffer->length());
 }
 
@@ -1249,17 +1249,17 @@ bool Protocol_text::store(Field *field)
   char buff[MAX_FIELD_WIDTH];
   String str(buff,sizeof(buff), &my_charset_bin);
   CHARSET_INFO *tocs= this->thd->variables.character_set_results;
-#ifndef DBUG_OFF
+#ifdef DBUG_ASSERT_EXISTS
   TABLE *table= field->table;
-  MY_BITMAP *old_map= 0;
+  my_bitmap_map *old_map= 0;
   if (table->file)
-    old_map= dbug_tmp_use_all_columns(table, &table->read_set);
+    old_map= dbug_tmp_use_all_columns(table, table->read_set);
 #endif
 
   field->val_str(&str);
-#ifndef DBUG_OFF
+#ifdef DBUG_ASSERT_EXISTS
   if (old_map)
-    dbug_tmp_restore_column_map(&table->read_set, old_map);
+    dbug_tmp_restore_column_map(table->read_set, old_map);
 #endif
 
   return store_string_aux(str.ptr(), str.length(), str.charset(), tocs);
@@ -1341,6 +1341,7 @@ bool Protocol_text::send_out_parameters(List<Item_param> *sp_params)
       continue;
     }
 
+    DBUG_ASSERT(sparam->get_item_param() == NULL);
     sparam->set_value(thd, thd->spcont, reinterpret_cast<Item **>(&item_param));
   }
 
