@@ -24,20 +24,6 @@ bool mysql_handle_derived(LEX *lex, uint phases);
 bool mysql_handle_single_derived(LEX *lex, TABLE_LIST *derived, uint phases);
 bool mysql_derived_reinit(THD *thd, LEX *lex, TABLE_LIST *derived);
 
-/**
-   Cleans up the SELECT_LEX_UNIT for the derived table (if any).
-
-   @param  thd         Thread handler
-   @param  lex         LEX for this thread
-   @param  derived     TABLE_LIST for the derived table
-
-   @retval  false  Success
-   @retval  true   Failure
-*/
-bool mysql_derived_cleanup(THD *thd, LEX *lex, TABLE_LIST *derived);
-
-Item *delete_not_needed_parts(THD *thd, Item *cond);
-
 bool pushdown_cond_for_derived(THD *thd, Item *cond, TABLE_LIST *derived);
 
 #endif /* SQL_DERIVED_INCLUDED */
