@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2015, 2021, MariaDB Corporation.
+Copyright (c) 2015, 2020, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -281,7 +281,7 @@ buf_read_ahead_random(
 
 	if (fil_space_t* space = fil_space_acquire(page_id.space())) {
 		high = space->max_page_number_for_io(high);
-		fil_space_release(space);
+		space->release();
 	} else {
 		return(0);
 	}
@@ -561,7 +561,7 @@ buf_read_ahead_linear(
 
 	if (fil_space_t* space = fil_space_acquire(page_id.space())) {
 		space_size = space->committed_size;
-		fil_space_release(space);
+		space->release();
 
 		if (high > space_size) {
 			/* The area is not whole */
@@ -789,7 +789,7 @@ buf_read_ibuf_merge_pages(
 					in the arrays */
 {
 #ifdef UNIV_IBUF_DEBUG
-	ut_a(n_stored < UNIV_PAGE_SIZE);
+	ut_a(n_stored < srv_page_size);
 #endif
 
 	for (ulint i = 0; i < n_stored; i++) {
@@ -806,21 +806,16 @@ tablespace_deleted:
 			continue;
 		}
 
-		ulint size = space->size;
-		if (!size) {
-			size = fil_space_get_size(space->id);
-		}
-
-		if (UNIV_UNLIKELY(page_nos[i] >= size)) {
+		if (UNIV_UNLIKELY(page_nos[i] >= space->size)) {
 			do {
 				ibuf_delete_recs(page_id_t(space_ids[i],
 							   page_nos[i]));
 			} while (++i < n_stored
 				 && space_ids[i - 1] == space_ids[i]
-				 && page_nos[i] >= size);
+				 && page_nos[i] >= space->size);
 			i--;
 next:
-			fil_space_release(space);
+			space->release();
 			continue;
 		}
 
@@ -847,7 +842,7 @@ next:
 		case DB_ERROR:
 			break;
 		case DB_TABLESPACE_DELETED:
-			fil_space_release(space);
+			space->release();
 			goto tablespace_deleted;
 		case DB_PAGE_CORRUPTED:
 		case DB_DECRYPTION_FAILED:
